/*
 * Copyright (c) 2004-2007 Erez Zadok
 * Copyright (c) 2004-2007 Stony Brook University
 * Copyright (c) 2004-2007 The Research Foundation of State University of New York
 * Copyright (c) 2004-2007 Nikolai Joukov
 * Copyright (c) 2006-2007 Gopala Surya
 *
 * For specific licensing information, see the COPYING file distributed with
 * this package, or get one from ftp://ftp.filesystems.org/pub/fist/COPYING.
 *
 * This Copyright notice must be kept intact and distributed with all
 * fistgen sources INCLUDING sources generated by fistgen.
 */

#ifndef _REPLAY_H
#define _REPLAY_H

/* Use smart memory buffering instead of LRU Management */
#define ORACLE_MM 1
/* Maximum size of memory buffers allowed */
#define MAX_MEMORY_BUFFERS_SIZE 16
#define PREFETCH 0
#define MULTITHREADING 0
#define REPLAY_TIMING 1
#define REPLAY_TIMING_SPEED 100
#define REPALY_TIMING_PRESPIN 1
/* This will disable replay of readpage(s) */
#define REPLAY_READS 0

#define __NR_sys_replay 230

#define MAX_RAT_SIZE 4096
/* reserved RAT fields */
#define RAT_ROOT_MNT 0
#define RAT_ROOT_DENTRY 1
/* their number */
#define MIN_RAT_SIZE 2

#define MAX_ARGS 6 

#ifdef __KERNEL__
#include <linux/smp_lock.h>
#include <asm/atomic.h>
#endif
#include "messages.h"

#define MAGIC_TRACE "TRFS"
#define MAGIC_RAT "RAT\0x1A"
#define MAGIC_CMDS "CMDS"

#define TRACE_VER 1

// trace file structures
struct trace_head {
	char magic[4];
	int  version;
};

struct trace_rat {
	char magic[4];
	int  num;
};

struct trace_cmd {
	char magic[4];
	int  num;
};

typedef struct timeval_s {
  unsigned long secs;
  unsigned long usecs;
} timeval_t;

typedef struct s_timeval_s {
  long secs;
  long usecs;
} s_timeval_t;

// replay entity
struct replay_command {
  op_t command;
  timeval_t time;
  int args[MAX_ARGS];
  int thread_loc;
  int ret;
  int expected_retval;
};

typedef struct kthread_data {
  struct semaphore *sem; //This is used to yield the thread
  struct replay* r;
  struct replay_command* srcmdp;
  struct task_struct *task;
  int initialized;
  int thread_loc;
  int terminate; //flag that signals to the thread that it should terminate
#ifdef __KERNEL__
  atomic_t *suspend;
#else
  void *suspend;
#endif
} kthread_data_t;

// resource types are defined in messages.h
// Resources allocation table
struct res_alloc_table {
  field_t type;
  union res {
    int	ival;
    char *cpval;
    struct file *file;
    struct dentry *dentry;
    struct inode *inode;
    struct vfsmount *mnt;
    struct page *page;
    struct kthread_data *kthread_data;
  }u;
  int	count;
};

#endif /* _REPLAY_H */
