/*
 * Copyright (c) 2004-2007 Erez Zadok
 * Copyright (c) 2004-2007 Stony Brook University
 * Copyright (c) 2004-2007 The Research Foundation of State University of New York
 * Copyright (c) 2004-2007 Nikolai Joukov
 *
 * For specific licensing information, see the COPYING file distributed with
 * this package, or get one from ftp://ftp.filesystems.org/pub/fist/COPYING.
 *
 * This Copyright notice must be kept intact and distributed with all
 * fistgen sources INCLUDING sources generated by fistgen.
 */

#ifndef __HASHED_HEAP_H
#define __HASHED_HEAP_H

#include <stdlib.h>
#include "hash.h"

typedef struct heap_entry_s heap_entry_t;

typedef struct hashed_heap_s {
  hash_t hash;
  heap_entry_t* heap_root;
  int (*compare_fn)(void*,int,void*,int); //key1,keysize_1,key2,keysize_2
} hashed_heap_t;

//Public Functions
int hashed_heap_init(hashed_heap_t *hashed_heap,int hash_size,int (*compare_fn)(void*,int,void*,int));
inline int hashed_heap_get_size(hashed_heap_t* hashed_heap);
int hashed_heap_add(hashed_heap_t* hashed_heap,void* key,int keylen,void* value);
void* hashed_heap_remove(hashed_heap_t* hashed_heap,void* key,int keylen);
void* hashed_heap_remove_root(hashed_heap_t* hashed_heap);
void* hashed_heap_get_root(hashed_heap_t* hashed_heap);
void* hashed_heap_find(hashed_heap_t* hashed_heap,void* key,int keylen);
int hashed_heap_destroy(hashed_heap_t* hashed_heap);

#endif
